/**
 * @module dixim
 * @submodule autoDownload
 */
dixim.namespace('dixim.autoDownload');

/**
 * @module autoDownload
 * @submodule auto-download-controller
 */

(function() {
var AutoDownloadController;
var prototype;
var superConstructor;
var AutoDownloadView = dixim.contents.AutoDownloadView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var UiFunctionManager = dixim.ctt.util.UiFunctionManager;
var Configs = dixim.ctt.Configs;
var ResourceBundle = dixim.ctt.ResourceBundle;
var ApiClientAutoDownload = dixim.common.ApiClientAutoDownload;
var OperationMenuDialogController = dixim.autoDownload.OperationMenuDialogController;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var SettingMenuDialogController = dixim.autoDownload.SettingMenuDialogController;
var Enum = dixim.ctt.util.Enum;

/**
 * @class AutoDownloadController
 * @constructor
 * @param {Object} configs
 */
AutoDownloadController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(AutoDownloadController, EventTarget);
dixim.autoDownload.AutoDownloadController = AutoDownloadController;
prototype = AutoDownloadController.prototype;
superConstructor = AutoDownloadController.superConstructor;

UiFunctionManager.set('autoDownload');

var SELECT = 'select';
var OPERATE = 'operate';
var REFRESH = 'refresh';
var PREV = 'prev';
var UPDATE = 'update';
var SETTING = 'setting';

var OperationMenu = Enum.fetch('OPERATION_MENU');

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'auto-download'].join('');

    ResourceBundle.reset();
    Configs.set('resourceBundlePath', 'lang/auto_download/');

    this._model = new ApiClientAutoDownload({isCancelAll: true});
    this._model.addEventListener('fetchTaskList', this._successFetchTaskList, this);
    this._model.addEventListener('fetchTaskListError', this._errorFetchTaskList, this);
    this._model.addEventListener('fetchFilterList', this._successFetchFilterList, this);
    this._model.addEventListener('fetchFilterListError', this._errorFetchFilterList, this);

    this._view = new AutoDownloadView(configs);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(REFRESH, this._clickRefresh, this);
    this._view.addEventListener(OPERATE, this._clickOperate, this);
    this._view.addEventListener(SETTING, this._clickSetting, this);
    this._view.addEventListener(PREV, this._clickPrev, this);

    this._parentContext = configs.parentContext;
    this._operationStatus = OperationMenu.NO_SELECTED;
};

/**
 * @method start
 */
prototype.start = function() {
    this._view.show();
    this._model.fetchTaskList();
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._successFetchTaskList = function(e) {
    var tasks = e.args.tasks;
    var tasksLen = tasks.length;

    if (tasksLen === 0) {
        this._view.noRow();
        this._view.checkActionBarStatus();
        return;
    }

    for (var i = 0; i < tasksLen; i += 1) {
        tasks[i].dstContainerPath = tasks[i].dstContainerPath.replace('/mnt/sataraid1/opt/dms-dtcp/exports/', '');
        this._view.addRow(tasks[i]);
    }
    this._view.checkActionBarStatus();
    this._model.fetchFilterList();
};

prototype._errorFetchTaskList = function(e) {
    this._view.error(_('download_auto_download_fetch_error'));
};

prototype._successFetchFilterList = function(e) {
    var filters = e.args.filters;
    var filtersLen = filters.length;
    var taskIds;
    var taskIdsLen;

    for (var i =  0; i < filtersLen; i += 1) {
        taskIds = filters[i].taskIds;
        taskIdsLen = taskIds.length;
        for (var j = 0; j < taskIdsLen; j += 1) {
            this._view.addFilter(taskIds[j], filters[i].filterName);
        }
    }
};

prototype._errorFetchFilterList = function() {
    this._errorDialog(_('auto_download_fetch_filter_error'));
};

prototype._clickRefresh = function() {
    this._update();
};

prototype._clickOperate = function(e) {
    var query = {};
    var controller;
    var taskList = [];
    var len;

    for (key in e.args.taskList) {
        taskList.push(e.args.taskList[key]);
    }

    this._operationStatus = OperationMenu.NO_SELECTED;
    len = taskList.length;
    if (len !== 0) this._operationStatus &= ~(OperationMenu.NO_SELECTED);
    if (len > 0) this._operationStatus |= OperationMenu.TASK;
    if (len > 1) this._operationStatus |= OperationMenu.MULTIPLE;

    query.operationStatus = this._operationStatus;
    query.taskList = taskList;
    controller = new OperationMenuDialogController(query);
    controller.addEventListener(UPDATE, this._update , this);
    controller.start();
};

prototype._clickSetting = function() {
    var controller;

    controller = new SettingMenuDialogController();
    controller.addEventListener(UPDATE, this._update , this);
    controller.start();
};

prototype._clickSelect = function() {
    this._view.switchSelect();
};

prototype._clickPrev = function() {
    this.stop();
    this._parentContext.start();
};

prototype._update = function() {
    this._view.cleanContent();
    this._model.fetchTaskList();
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('download_menu_auto_download');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

}());
