/**
 * @module ctt
 * @submodule dms-listing-controller
 */
(function() {
var Event = dixim.event.Event;
var EventTarget = dixim.event.EventTarget;
var DestinationStorageListingView = dixim.ctt.DestinationStorageListingView;

var DestinationStorageListingController;
var prototype;

/**
 * @class DestinationStorageListingController
 * @constructor
 * @param {Object} configs
 */
DestinationStorageListingController = function(configs) {
  EventTarget.call(this);
  this._initialize(configs);
};
dixim.inherit(DestinationStorageListingController, EventTarget);
dixim.ctt.DestinationStorageListingController = DestinationStorageListingController;
prototype = DestinationStorageListingController.prototype;

/**
 * Item event is sent when the selection is changes.
 * 
 * @event changeSelection
 */
var CHANGE_SELECTION = 'changeSelection';

/**
 * Item event is sent when the list is updated.
 * 
 * @event updateList
 */
var UPDATE_LIST = 'updateList';

/**
 * @event 
 */
var UPDATE_LIST_ERROR = 'updateListError';

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
  this._containerNode = configs.containerNode;
  this._model = configs.model;
  this._view = new DestinationStorageListingView({containerNode: this._containerNode});
  this._currentDms = configs.dms || null;
  this._session = null;
  this._records = [];
};

/**
 * @method start
 */
prototype.start = function() {
  this._view.addEventListener('render', this._didRenderView, this);
  this._view.addEventListener('clickItem', this._didClickItem, this);
  this._view.addEventListener('changeSelection', this._didChangeSelection, this);
  this._model.addEventListener('recDestList', this._didFetchRecDestList, this);
  this._model.addEventListener('recDestListError', this._didFailFetchRecDestList, this);

  this._view.show();
};

prototype._didRenderView = function(e) {
  if (this._currentDms) {
    this._updateCurrentDms(this._currentDms);
  }
};

prototype._didClickItem = function(e) {
  var record = e.args.record;

  if (record.availableCapacity > 0) {
    this._view.selectRow(record);
  }
};

prototype._didFetchRecDestList = function(e) {
  var session = e.args.session;

  if (session !== this._session) return;
  this._session = null;
  if (session.isCancelled()) return;

  this._records = e.args.data.results;

  if (this._records[0].availableCapacity === 0) {
    this._records.push({});
  }

  this._view.removeAllRows();
  this._view.addRows(this._records);
  this.dispatchEvent(new Event(UPDATE_LIST));
};

prototype._didFailFetchRecDestList = function(e) {
  var session = e.args.session;

  if (session !== this._session) return;
  this._session = null;
  this._view.removeAllRows();
  this._records = [];
  this.dispatchEvent(new Event(UPDATE_LIST_ERROR));
};

prototype._didChangeSelection = function(e) {
  this.dispatchEvent(new Event(CHANGE_SELECTION));
};

prototype.updateCurrentDms = function(dms) {
  this._updateCurrentDms(dms);
};

prototype._updateCurrentDms = function(dms) {
  if (!!this._session) {
    this._session.cancel();
  }

  this._view.removeAllRows();
  this._currentDms = dms;

  if (!!dms) {
    this._session = this._model.recDestList({udn: dms.udn});
  }
};

prototype.getSelectedStorage = function(e) {
  var list;

  list = this._view.getSelectedRows();
  return list.length ? list[0] : null;
};

prototype.getNumberOfStorages = function() {
  return this._records.length;
};

prototype.stop = function() {
  if (!!!this._view) return;
  this._view.hide();
};

prototype.destroy = function() {
  if (!!!this._view) return;
  this._view.destroy();
};

prototype.selectStorage = function(storage) {
  var records = this._records;
  var len = records.length;
  var i;

  for (i = 0; i < len; i += 1) {
    if (storage.id === records[i].id) {
      this._selectStorage(records[i]);
      break;
    }
  }
};

prototype._selectStorage = function(storage) {
  this._view.selectRow(storage);
};

prototype.selectFirstStorage = function() {
  var records = this._records;

  if (records.length) {
    this._selectStorage(records[0]);
  }
};

}());
